@@ -16,15 +16,17 @@ class DiffReq(object):
 
     def __init__(self, name, vers):
         self.name = name
-        # ensure that comp. operators are surrounded with spaces
-        vers = re.sub(r'([<>=!]+)', r'\1 ', vers)
+        # ensure there is only one space between comp. operator and version
+        vers = re.sub(r'([<>=!]+)', r'\1 ', vers.replace(" ", ""))
         self.vers = vers
         self.old_vers = vers
 
-    def __str__(self):
+    def __str__(self, format=None):
         s = '%s %s' % (self.name, self.vers)
         if self.old_vers != self.vers:
             s += '  (was %s)' % (self.old_vers or 'not capped',)
+        if not format:
+            pass
         return s
 
 
@@ -36,15 +38,17 @@ class CheckReq(object):
         self.vers = vers
 
     def met(self):
-        if self.vers is None:
+        for rv in self.desired_vers.split(','):
+            m = re.match('(>|>=) (\\d.*)$', rv)
+            if m:
+                if rv == self.vers:
+                    return True
+                else:
+                    return False
+        if not self.desired_vers and not self.vers:
+            return True
+        else:
             return False
-        # TODO: smarter version rage comparison
-        if self.desired_vers:
-            if self.desired_vers == self.vers:
-                return True
-            else:
-                return False
-        return True
 
     def __str__(self, format=None):
         s = self.name
@@ -55,36 +59,104 @@ class CheckReq(object):
         return s
 
 
-def parse_reqs_txt(txt):
+def is_dependency_should_be_checked(python_version, environment_markers):
+    env_markers = re.sub(r'[;\'"]', '', environment_markers.replace(" ", ""))
+    env_markers = env_markers.split('or')
+    should_be_checked = list()
+    for marker in env_markers:
+        if 'python_version' in marker:
+            marker_version = marker.replace('python_version', '')
+            mvr = match_version_rule(python_version, marker_version)
+            if mvr == VER_OK:
+                should_be_checked.append(True)
+            else:
+                should_be_checked.append(False)
+    if len(should_be_checked) != 0 and True not in should_be_checked:
+        return False
+    return True
+
+
+def parse_reqs_txt(txt, py_vers='3.6'):
     reqs = []
+    if not txt:
+        log.warn("The requirements.txt file is empty")
+        return reqs
     lines = sorted(txt.split('\n'), key=lambda l: l.lower())
     for line in lines:
-        if not line or re.match('\W', line):
+        # removing comments and spaces
+        line = re.sub(r'\s*(?:#.*)$', '', line.replace(" ", ""))
+        if not line:
             continue
-        line = re.sub(r'\s*(?:#.*)$', '', line)
-        m = re.match(r'([^<>=!\s]+)\s*(.*)$', line)
-        if not m:
+        m = re.match(r'([\w\._-]+)([<>=!,.\d]*)(;.*)?$', line)
+        if m:
+            dep_name, dep_version, env_markers = m.groups()
+            if env_markers is None:
+                r = DiffReq(name=dep_name, vers=dep_version)
+                reqs.append(r)
+            else:
+                if is_dependency_should_be_checked(py_vers, env_markers):
+                    r = DiffReq(name=dep_name, vers=dep_version)
+                    reqs.append(r)
+        else:
             log.warn("Failed to parse requirement: %s" % line)
-            continue
-        r = DiffReq(name=m.group(1), vers=m.group(2))
-        reqs.append(r)
     return reqs
 
 
 def get_reqs_from_ref(ref):
-    o = git('show', '%s:requirements.txt' % ref, log_cmd=False)
-    return parse_reqs_txt(o)
+    try:
+        o = git('show', '%s:requirements.txt' % ref, log_cmd=False)
+        return parse_reqs_txt(o)
+    except Exception:
+        return ''
 
 
-def get_reqs_from_path(path):
+def get_upstream_first_ref_of_stable_releases_before(timestamp=None):
+    upstream = list()
+    for branch in git.remote_branches():
+        if 'upstream/stable' in branch:
+            commits = git.get_commits_of_branch(branch)
+            try:
+                if (commits[0]['timestamp'] < timestamp or timestamp is None):
+                    upstream.append(commits[0]['hash'])
+            except IndexError:
+                pass
+    return upstream
+
+
+def get_reqs_present_in_upstream_before(timestamp, py_version='3.6'):
+    reqs = list()
+    for ref in get_upstream_first_ref_of_stable_releases_before(timestamp):
+        reqs += get_reqs_from_ref(ref, py_version)
+    return reqs
+
+
+def get_pkgs_present_in_upstream_before(timestamp, py_version='3.6'):
+    pkgs = list()
+    reqs_txt = get_reqs_present_in_upstream_before(timestamp, py_version)
+    map_reqs2pkgs(reqs_txt, 'epel')
+    for _pkg in reqs_txt:
+        if _pkg.name not in pkgs:
+            pkgs.append(_pkg.name)
+    return pkgs
+
+
+def get_reqs_from_ref(ref, py_version='3.6'):
+    try:
+        o = git('show', '%s:requirements.txt' % ref, log_cmd=False)
+    except Exception:
+        o = ''
+    return parse_reqs_txt(o, py_version)
+
+
+def get_reqs_from_path(path, py_version='3.6'):
     o = open(path).read()
-    return parse_reqs_txt(o)
+    return parse_reqs_txt(o, py_version)
 
 
 def get_reqs_from_spec(as_objects=False, normalize_py23=False):
     spec = specfile.Spec()
-    reqs = spec.get_requires(versions_as_string=True,
-                             normalize_py23=normalize_py23)
+    reqs = spec.get_requires_not_provided(versions_as_string=True,
+                                          normalize_py23=normalize_py23)
     if as_objects:
         creqs = []
         for name in sorted(reqs):
@@ -121,9 +193,9 @@ def reqdiff(reqs1, reqs2):
     return added, changed, removed
 
 
-def reqdiff_from_refs(ref1, ref2):
-    r1 = get_reqs_from_ref(ref1)
-    r2 = get_reqs_from_ref(ref2)
+def reqdiff_from_refs(ref1, ref2, py_version):
+    r1 = get_reqs_from_ref(ref1, py_version)
+    r2 = get_reqs_from_ref(ref2, py_version)
     return reqdiff(r1, r2)
 
 
@@ -143,32 +215,56 @@ def print_reqdiff(added, changed, removed):
 
 
 def reqcheck(desired_reqs, reqs):
-    met, any_version, wrong_version, missing = [], [], [], []
+    met, any_version, wrong_version, missing, removed = [], [], [], [], []
+    excess = reqs
     for dr in desired_reqs:
-        vers = reqs.get(dr.name)
-        r = CheckReq(dr.name, dr.vers, vers)
-        if r.vers is None:
-            missing.append(r)
-        elif r.vers is '':
-            if r.desired_vers:
-                any_version.append(r)
+        for req in reqs:
+            if req.name == dr.name:
+                r = CheckReq(dr.name, dr.vers, req.vers)
+                try:
+                    excess.remove(req)
+                except Exception as ex:
+                    pass
+                break
+        try:
+            if r.vers is None:
+                missing.append(r)
+            elif not r.vers:
+                if r.desired_vers:
+                    any_version.append(r)
+                else:
+                    met.append(r)
             else:
-                met.append(r)
-        else:
-            if r.met():
-                met.append(r)
-            else:
-                wrong_version.append(r)
+                if r.met():
+                    met.append(r)
+                else:
+                    wrong_version.append(r)
+            del r
 
-    return met, any_version, wrong_version, missing
+        except Exception as ex:
+            pass
+
+    current_commit = git.current_commit()
+    current_commit_timestamp = git.get_timestamp_by_ref(current_commit)
+    pkgs_used_in_the_past = get_pkgs_present_in_upstream_before(
+        current_commit_timestamp)
+    for i, excess_pkg in enumerate(excess):
+        if excess_pkg.name in pkgs_used_in_the_past:
+            removed.append(excess_pkg)
+            del excess[i]
+
+    return met, any_version, wrong_version, missing, excess, removed
 
 
-def print_reqcheck(met, any_version, wrong_version, missing, format=None):
+def print_reqcheck(met, any_version, wrong_version, missing, excess, removed,
+                   format=None):
     cats = [
         ("{t.bold_green}MET{t.normal}:", met),
         ("{t.bold}VERSION NOT ENFORCED{t.normal}:", any_version),
+        ("{t.bold}ADDITIONAL REQUIRES{t.normal}:", excess),
         ("{t.bold_yellow}VERSION MISMATCH{t.normal}:", wrong_version),
         ("{t.bold_red}MISSING{t.normal}:", missing),
+        ("{t.bold_red}REMOVED{t.normal}:", removed),
     ]
     if format == 'spec':
         # get alignment from .spec file
@@ -189,16 +285,16 @@ def print_reqcheck(met, any_version, wrong_version, missing, format=None):
         helpers.print_list(reqs, pre=pre)
 
 
-def reqcheck_spec(ref=None, reqs_txt=None):
+def reqcheck_spec(py_version, ref=None, reqs_txt=None):
     if (ref and reqs_txt) or (not ref and not reqs_txt):
         raise exception.InvalidUsage(
             why="reqcheck_spec needs either ref (git ref) or reqs_txt (path)")
     if ref:
-        reqs_txt = get_reqs_from_ref(ref)
+        reqs_txt = get_reqs_from_ref(ref, py_version)
     else:
-        reqs_txt = get_reqs_from_path(reqs_txt)
+        reqs_txt = get_reqs_from_path(reqs_txt, py_version)
     map_reqs2pkgs(reqs_txt, 'epel')
-    spec_reqs = get_reqs_from_spec(normalize_py23=True)
+    spec_reqs = get_reqs_from_spec(as_objects=True, normalize_py23=True)
     return reqcheck(reqs_txt, spec_reqs)
 
 
