@@ -16,6 +16,16 @@ class RdopkgException(Exception):
         super(RdopkgException, self).__init__(msg)
 
 
+class ReqCheckMissingDependencies(RdopkgException):
+    msg_fmt = "There are missing dependencies"
+    exit_code = 2
+
+
+class ReqCheckMismatchingDependencies(RdopkgException):
+    msg_fmt = "There are mismatching dependencies"
+    exit_code = 4
+
+
 class CommandFailed(RdopkgException):
     msg_fmt = "Command failed: %(cmd)s"
 
@@ -215,3 +225,7 @@ class InvalidLintCheck(RdopkgException):
 class LintProblemsFound(RdopkgException):
     exit_code = 23
     msg_fmt = "Lint problems detected."
+
+
+class WrongPythonVersion(RdopkgException):
+    msg_fmt = "The python version specified is wrong (set '3.6' for python3.6)"
