@@ -111,6 +111,8 @@ def run(action_runner, cargs, prog='rdopkg', version=None):
             exception.RequiredActionArgumentNotAvailable,
             exception.RequirementNotMet,
             exception.RpmModuleNotAvailable,
+            exception.ReqCheckMismatchingDependencies,
+            exception.ReqCheckMissingDependencies,
             exception.SpecFileNotFound,
             exception.ToolNotFound,
             exception.UserAbort,
